# A quick hack for testing the relative speeds of different
# implementations of the Mandelbrot calculator.

# Run this script with 

#     ipython --pylab -i compare_speed.py

# Specify which versions you want to compare here
versions = '345'
# This should be a Python iterable: if your version numbers go into
# double digits (or you use some other multiple-character version
# identifiers) you'll need to replace this with something like:
#
#versions = '3 4 5 6 7 8 9 10 11 12'.split()

# ToDo: make it configurable with command line options.

from sys import argv

# If your timings take too long or too short, try changing some of
# these values, especially resolution and max_iterations.
params = dict(resolution = 500,
              x = -0.75,
              y = 0,
              width = 2.5,
              max_iterations = 100)

mandel = {}
timings = {}

for version in versions:
    # Import modules whose names are not known at compile time. In
    # newer versions of Python, use the importlib module instead of
    # __import__
    module = __import__('mandelbrot_'+version)
    # Because the Mandelbrot implementations were hacked together
    # quickly and rely on being run inside IPython's pylab global
    # environment, some names are missing from their modules' global
    # scope when the different implementations are imported as
    # modules. Here is a hack to ensure that the required names are
    # present in those modules.
    for name in 'figure ogrid empty ndenumerate imshow'.split():
        setattr(module, name, globals()[name])
    # Create an instance of each version of the Mandelbrot
    # implementation that is being timed.
    mandel[version] = module.Mandelbrot(**params)
    # Collect the timings
    timings[version] = mandel[version].last_calculation_time

sorted_timings = sorted(timings.items(), key=lambda pair:pair[1], reverse=True)
slowest = sorted_timings[0][1]
scaled_timings = [ (v, slowest/t) for (v,t) in sorted_timings ]
print "version speedup"
for v,t in scaled_timings:
    print "  %s  %10.1f" % (v, t)
